# 快速开始


``` python
! [ -e /content ] && pip install -Uqq fastai  # 在Colab上升级fastai
```

``` python
from fastai.vision.all import *
from fastai.text.all import *
from fastai.collab import *
from fastai.tabular.all import *
```

fastai 的应用程序都使用相同的基本步骤和代码：

- 创建合适的 `DataLoaders`
- 创建一个 `Learner`
- 调用 *fit* 方法
- 进行预测或查看结果。

在这个快速入门中，我们将展示这些步骤适用于广泛的不同应用和数据集。正如您所看到的，尽管使用了非常不同的模型和数据，每种情况的代码非常相似。

## 计算机视觉分类

以下代码完成了以下任务：

1.  从 fast.ai 数据集集合中下载一个名为 [Oxford-IIIT
    宠物数据集](http://www.robots.ox.ac.uk/~vgg/data/pets/)
    的数据集，该数据集包含 7,349 张来自 37
    个不同品种的猫和狗的图像，并将其提取到您正在使用的 GPU 服务器上。
2.  从互联网上下载一个 *预训练模型*，该模型已经在 130
    万张图像上训练过，使用的是一款获奖模型。
3.  利用最新的迁移学习进展对预训练模型进行
    *微调*，以创建一个特别定制用于识别狗和猫的模型。

前两个步骤只需运行一次。如果您再次运行，将使用已经下载的数据集和模型，而不是重新下载它们。

``` python
path = untar_data(URLs.PETS)/'images'

def is_cat(x): return x[0].isupper()
dls = ImageDataLoaders.from_name_func(
    path, get_image_files(path), valid_pct=0.2, seed=42,
    label_func=is_cat, item_tfms=Resize(224))

learn = vision_learner(dls, resnet34, metrics=error_rate)
learn.fine_tune(1)
```

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">error_rate</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>0.173790</td>
<td>0.018827</td>
<td>0.005413</td>
<td>00:12</td>
</tr>
</tbody>
</table>

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">error_rate</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>0.064295</td>
<td>0.013404</td>
<td>0.005413</td>
<td>00:14</td>
</tr>
</tbody>
</table>

您可以使用`predict`方法对您的模型进行推断：

``` python
img = PILImage.create('images/cat.jpg')
img
```

![](quick_start_files/figure-commonmark/cell-5-output-1.png)

``` python
is_cat,_,probs = learn.predict(img)
print(f"Is this a cat?: {is_cat}.")
print(f"Probability it's a cat: {probs[1].item():.6f}")
```

    Is this a cat?: True.
    Probability it's a cat: 0.999722

### 计算机视觉分割

以下是我们如何使用[*Camvid*
数据集](http://www0.cs.ucl.ac.uk/staff/G.Brostow/papers/Brostow_2009-PRL.pdf)的一个子集，利用fastai训练分割模型的方法：

``` python
path = untar_data(URLs.CAMVID_TINY)
dls = SegmentationDataLoaders.from_label_func(
    path, bs=8, fnames = get_image_files(path/"images"),
    label_func = lambda o: path/'labels'/f'{o.stem}_P{o.suffix}',
    codes = np.loadtxt(path/'codes.txt', dtype=str)
)

learn = unet_learner(dls, resnet34)
learn.fine_tune(8)
```

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>2.882460</td>
<td>2.096923</td>
<td>00:03</td>
</tr>
</tbody>
</table>

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>1.602270</td>
<td>1.543582</td>
<td>00:02</td>
</tr>
<tr class="even">
<td>1</td>
<td>1.417732</td>
<td>1.225782</td>
<td>00:02</td>
</tr>
<tr class="odd">
<td>2</td>
<td>1.307454</td>
<td>1.071090</td>
<td>00:02</td>
</tr>
<tr class="even">
<td>3</td>
<td>1.170338</td>
<td>0.884501</td>
<td>00:02</td>
</tr>
<tr class="odd">
<td>4</td>
<td>1.047036</td>
<td>0.799820</td>
<td>00:02</td>
</tr>
<tr class="even">
<td>5</td>
<td>0.947965</td>
<td>0.754801</td>
<td>00:02</td>
</tr>
<tr class="odd">
<td>6</td>
<td>0.868178</td>
<td>0.728161</td>
<td>00:02</td>
</tr>
<tr class="even">
<td>7</td>
<td>0.804939</td>
<td>0.720942</td>
<td>00:02</td>
</tr>
</tbody>
</table>

我们可以通过要求模型为图像的每个像素上色，以可视化它完成任务的效果。

``` python
learn.show_results(max_n=6, figsize=(7,8))
```

![](quick_start_files/figure-commonmark/cell-8-output-2.png)

或者，我们可以使用`SegmentationInterpretation`类绘制对验证损失贡献最大的`k`个实例。

``` python
interp = SegmentationInterpretation.from_learner(learn)
interp.plot_top_losses(k=2)
```

![](quick_start_files/figure-commonmark/cell-9-output-2.png)

## 自然语言处理

以下是训练一个模型所需的所有代码，该模型能够对电影评论的情感进行分类，优于五年前任何存在的模型：

``` python
dls = TextDataLoaders.from_folder(untar_data(URLs.IMDB), valid='test')
learn = text_classifier_learner(dls, AWD_LSTM, drop_mult=0.5, metrics=accuracy)
learn.fine_tune(2, 1e-2)
```

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">accuracy</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>0.594912</td>
<td>0.407416</td>
<td>0.823640</td>
<td>01:35</td>
</tr>
</tbody>
</table>

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">accuracy</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>0.268259</td>
<td>0.316242</td>
<td>0.876000</td>
<td>03:03</td>
</tr>
<tr class="even">
<td>1</td>
<td>0.184861</td>
<td>0.246242</td>
<td>0.898080</td>
<td>03:10</td>
</tr>
<tr class="odd">
<td>2</td>
<td>0.136392</td>
<td>0.220086</td>
<td>0.918200</td>
<td>03:16</td>
</tr>
<tr class="even">
<td>3</td>
<td>0.106423</td>
<td>0.191092</td>
<td>0.931360</td>
<td>03:15</td>
</tr>
</tbody>
</table>

预测是通过 `predict` 完成的，正如在计算机视觉中一样：

``` python
learn.predict("I really liked that movie!")
```

    ('pos', tensor(1), tensor([0.0041, 0.9959]))

## 表格

从普通的*表格*数据构建模型的基本步骤与之前的模型相同。以下是训练一个模型所需的代码，该模型将根据个人的社会经济背景预测他们是否为高收入者：

``` python
path = untar_data(URLs.ADULT_SAMPLE)

dls = TabularDataLoaders.from_csv(path/'adult.csv', path=path, y_names="salary",
    cat_names = ['workclass', 'education', 'marital-status', 'occupation',
                 'relationship', 'race'],
    cont_names = ['age', 'fnlwgt', 'education-num'],
    procs = [Categorify, FillMissing, Normalize])

learn = tabular_learner(dls, metrics=accuracy)
learn.fit_one_cycle(2)
```

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">accuracy</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>0.372298</td>
<td>0.359698</td>
<td>0.829392</td>
<td>00:06</td>
</tr>
<tr class="even">
<td>1</td>
<td>0.357530</td>
<td>0.349440</td>
<td>0.837377</td>
<td>00:06</td>
</tr>
</tbody>
</table>

## 推荐系统

推荐系统非常重要，特别是在电子商务领域。像亚马逊和Netflix这样的公司努力推荐用户可能喜欢的产品或电影。以下是如何训练一个模型，根据用户以前的观看习惯预测他们可能喜欢的电影，使用[MovieLens数据集](https://doi.org/10.1145/2827872)：

``` python
path = untar_data(URLs.ML_SAMPLE)
dls = CollabDataLoaders.from_csv(path/'ratings.csv')
learn = collab_learner(dls, y_range=(0.5,5.5))
learn.fine_tune(6)
```

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>1.497551</td>
<td>1.435720</td>
<td>00:00</td>
</tr>
</tbody>
</table>

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: left;">
<th data-quarto-table-cell-role="th">epoch</th>
<th data-quarto-table-cell-role="th">train_loss</th>
<th data-quarto-table-cell-role="th">valid_loss</th>
<th data-quarto-table-cell-role="th">time</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>0</td>
<td>1.332337</td>
<td>1.351769</td>
<td>00:00</td>
</tr>
<tr class="even">
<td>1</td>
<td>1.180177</td>
<td>1.046801</td>
<td>00:00</td>
</tr>
<tr class="odd">
<td>2</td>
<td>0.913091</td>
<td>0.799319</td>
<td>00:00</td>
</tr>
<tr class="even">
<td>3</td>
<td>0.749806</td>
<td>0.731218</td>
<td>00:00</td>
</tr>
<tr class="odd">
<td>4</td>
<td>0.686577</td>
<td>0.715372</td>
<td>00:00</td>
</tr>
<tr class="even">
<td>5</td>
<td>0.665683</td>
<td>0.713309</td>
<td>00:00</td>
</tr>
</tbody>
</table>

我们可以使用之前看到的相同的 `show_results`
调用来查看一些用户和电影ID、实际评分和预测的示例：

``` python
learn.show_results()
```

<table class="dataframe" data-quarto-postprocess="true" data-border="1">
<thead>
<tr class="header" style="text-align: right;">
<th data-quarto-table-cell-role="th"></th>
<th data-quarto-table-cell-role="th">userId</th>
<th data-quarto-table-cell-role="th">movieId</th>
<th data-quarto-table-cell-role="th">rating</th>
<th data-quarto-table-cell-role="th">rating_pred</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td data-quarto-table-cell-role="th">0</td>
<td>5.0</td>
<td>3.0</td>
<td>2.0</td>
<td>3.985477</td>
</tr>
<tr class="even">
<td data-quarto-table-cell-role="th">1</td>
<td>1.0</td>
<td>62.0</td>
<td>4.0</td>
<td>3.629225</td>
</tr>
<tr class="odd">
<td data-quarto-table-cell-role="th">2</td>
<td>91.0</td>
<td>81.0</td>
<td>1.0</td>
<td>3.476280</td>
</tr>
<tr class="even">
<td data-quarto-table-cell-role="th">3</td>
<td>48.0</td>
<td>26.0</td>
<td>2.0</td>
<td>4.043919</td>
</tr>
<tr class="odd">
<td data-quarto-table-cell-role="th">4</td>
<td>75.0</td>
<td>54.0</td>
<td>3.0</td>
<td>4.023057</td>
</tr>
<tr class="even">
<td data-quarto-table-cell-role="th">5</td>
<td>42.0</td>
<td>22.0</td>
<td>3.0</td>
<td>3.509050</td>
</tr>
<tr class="odd">
<td data-quarto-table-cell-role="th">6</td>
<td>40.0</td>
<td>59.0</td>
<td>4.0</td>
<td>3.686552</td>
</tr>
<tr class="even">
<td data-quarto-table-cell-role="th">7</td>
<td>63.0</td>
<td>77.0</td>
<td>3.0</td>
<td>2.862713</td>
</tr>
<tr class="odd">
<td data-quarto-table-cell-role="th">8</td>
<td>32.0</td>
<td>61.0</td>
<td>4.0</td>
<td>4.356578</td>
</tr>
</tbody>
</table>

``` python

```
